local api = vim.api

local saga = require 'lspsaga'
local lspcfg = require 'lspconfig'
local compe = require 'compe'

-- Enable the following language servers
local servers = {
    'bashls', 'gopls', 'texlab', 'pyright', 'rust_analyzer', 'tsserver', 'vimls'
    -- sumneko_lua further down, needs more setup
}

vim.o.completeopt = "menuone,noselect"
-- completion sources, higher priority = closer to the top?
compe.setup({
    source = {
        nvim_lsp = {kind = ""},
        nvim_treesitter = {kind = "⧻"},
        buffer = {kind = ""},
        path = {kind = ""},
        calc = {kind = ""},
        nvim_lua = true,
        emoji = {
            kind = "ﲃ",
            filetypes = {"markdown", "text", "pandoc", "rst", "asciidoc"}
        },
        latex_symbols = {
            kind = "λ",
            filetypes = {"markdown", "text", "pandoc", "rst", "asciidoc"}
        },
        tmux = {kind = "τ"},
        spell = {kind = "", priority = 0},
        vsnip = true
    }
})

local capabilities = vim.lsp.protocol.make_client_capabilities()
capabilities.textDocument.completion.completionItem.snippetSupport = true
capabilities.textDocument.completion.completionItem.resolveSupport = {
    properties = {'documentation', 'detail', 'additionalTextEdits'}
}

local on_attach = function(_, _)
    -- Keybindings for LSPs
    -- Note these are in on_attach so that they don't override bindings in a non-LSP setting
    api.nvim_set_keymap("n", "gh",
                        "<cmd>lua require 'lspsaga.provider'.lsp_finder()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "gd",
                        "<cmd>lua require'lspsaga.provider'.preview_definition()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "gE",
                        "<cmd>lua require 'lspsaga.codeaction'.code_action()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("v", "gE",
                        "<cmd>'<,'>lua require 'lspsaga.codeaction'.range_code_action()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "K",
                        "<cmd>lua require('lspsaga.hover').render_hover_doc()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "gK",
                        "<cmd>lua require('lspsaga.signaturehelp').signature_help()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "gr",
                        "<cmd>lua require('lspsaga.rename').rename()<CR>",
                        {noremap = true, silent = true})

    api.nvim_set_keymap("n", "ge",
                        "<cmd>lua require('lspsaga.diagnostic').show_line_diagnostics()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "]e",
                        "<cmd>lua require('lspsaga.diagnostic').lsp_jump_diagnostic_next()<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "[e",
                        "<cmd>lua require('lspsaga.diagnostic').lsp_jump_diagnostic_prev()<CR>",
                        {noremap = true, silent = true})

    api.nvim_set_keymap("n", "C-f",
                        "<cmd>lua require('lspsaga.action').smart_scroll_with_saga(1)<CR>",
                        {noremap = true, silent = true})
    api.nvim_set_keymap("n", "C-b",
                        "<cmd>lua require('lspsaga.action').smart_scroll_with_saga(-1)<CR>",
                        {noremap = true, silent = true})

    require("lspsaga").init_lsp_saga {
        error_sign = 'X',
        warn_sign = '⚠️',
        hint_sign = '⚡',
        infor_sign = 'ℹ️',
        code_action_icon = '●',
        finder_definition_icon = '📖 ',
        finder_reference_icon = '🔖 ',
        definition_preview_icon = '📖 ',
        finder_action_keys = {
            open = '<cr>',
            split = 's',
            vsplit = 'v',
            quit = '<esc>',
            scroll_down = '<c-f>',
            scroll_up = '<c-b>'
        },
        code_action_keys = {quit = '<esc>', exec = '<cr>'},
        rename_action_keys = {quit = '<esc>', exec = '<cr>'}
    }
    vim.cmd("command! LspHover lua vim.lsp.buf.hover()<CR>")
    vim.cmd(
        "command! LspDisable lua vim.lsp.stop_client(vim.lsp.get_active_clients())<CR>")
    print('LSP ready')
end

-- set up simple servers
for _, lsp in ipairs(servers) do lspcfg[lsp].setup {on_attach = on_attach} end

lspcfg.efm.setup {
    on_attach = on_attach,
    init_options = {
        documentFormatting = true,
        codeAction = true,
        completion = true,
        documentSymbol = true,
        hover = true
    },
    filetypes = {"sh"},
    settings = {
        rootMarkers = {".git/"},
        languages = {
            sh = {
                {
                    lintCommand = 'shellcheck -f gcc -x',
                    lintFormats = {
                        '%f:%l:%c: %trror: %m', '%f:%l:%c: %tarning: %m',
                        '%f:%l:%c: %tote: %m'
                    }
                }, {formatCommand = 'shfmt -ci -s -bn', formatStdin = true}
            }
        }
    }
}

-- requires the lua-language-server package to be installed
-- The arch package defaults to the following directory
local sumneko_root_path = "/usr/share/lua-language-server"
lspcfg.sumneko_lua.setup {
    cmd = {"lua-language-server", "-E", sumneko_root_path .. "/main.lua"},
    settings = {
        Lua = {
            runtime = {
                -- Tell the language server which version of Lua you're using (most likely LuaJIT in the case of Neovim)
                version = 'LuaJIT',
                -- Setup your lua path
                path = vim.split(package.path, ';')
            },
            diagnostics = {
                -- Get the language server to recognize additional globals
                globals = {
                    'vim', 'before_each', 'after_each', 'describe', 'it',
                    'mock', 'stub'
                }
            },
            workspace = {
                -- Make the server aware of additional runtime files
                library = {
                    [vim.fn.expand('$VIMRUNTIME/lua')] = true,
                    [vim.fn.expand('$VIMRUNTIME/lua/vim/lsp')] = true,
                    ["/usr/share/lua/5.1/busted/"] = true
                }
            }
        }
    },
    on_attach = on_attach
}

saga.init_lsp_saga()
